/*
	$Id: Display.c 217 2006-05-29 09:39:55Z syl $

	Copyright (C) 2006 Syllabub
	Maroontress Fast Software.
*/

#include <sys/types.h>
#include <sys/param.h>
#include <netinet/in.h>
#include <stdio.h>
#include <string.h>
#include <err.h>
#include "Socket.h"
#include "Event.h"
#include "InputStream.h"
#include "BufferedInputStream.h"
#include "FileDescriptor.h"
#include "Pool.h"
#include "Recipe.h"
#include "OutputStream.h"
#include "Display.h"

#define DEFAULT_PORTNAME "30000"

static void
pourRecipe(union Pool pool, const char *const *av)
{
    union OutputStream out;

    if ((out = Pool_getOutputStream(pool)).super.object == NULL)
	errx(1, "Pool_getOutputSteram() failed.");
    Recipe_print(out, av);
    OutputStream_close(out);
}

static void
drainRecipe(union Pool pool, union OutputStream out)
{
    union InputStream in;
    char buf[4096];
    ssize_t size;

    if ((in = Pool_getInputStream(pool)).super.object == NULL)
	errx(1, "Pool_getInputSteram() failed.");
    /*
      μǤϥ򥳥ԡ뤳ȤˤʤΤǡPool_drainTo(out)
      Τ褦ʥ᥽åɤߤ
    */
    while ((size = InputStream_read(in, buf, sizeof(buf))) > 0) {
	if (OutputStream_writeFully(out, buf, size) < 0) {
	    errx(1, "OutputStream_writeFully() failed.");
	}
    }
    InputStream_close(in);
}

static void
handshake(union OutputStream out, const char *const *av)
{
    union Pool pool;
    long size;

    if ((pool = Pool_create()).super.object == NULL)
	errx(1, "Pool_create() failed.");
    pourRecipe(pool, av);

    size = Pool_getSize(pool);
    size = htonl(size);
    if (OutputStream_writeFully(out, &size, sizeof(size)) < 0) {
	errx(1, "OutputStream_writeFully() failed.");
    }

    drainRecipe(pool, out);
    Pool_free(pool);
}

struct Display {
    int fd;
    union Event *lastEvent;
    union BufferedInputStream in;
    union OutputStream out;
    const char *const *testsuite;
};

static int
openSocket(const char *hostport)
{
    char *displayName, *colon;
    const char *portName;
    int fd;

    if (hostport == NULL)
	hostport = "127.0.0.1";
    if ((displayName = strdup(hostport)) == NULL)
	err(1, "strdup");
    if ((colon = strchr(displayName, ':')) != NULL) {
	*colon = 0;
	portName = colon + 1;
    }
    else {
	portName = DEFAULT_PORTNAME;
    }
    fd = Socket_open(displayName, portName);
    free(displayName);
    return fd;
}

struct Display *
Display_open(const char *hostport, const char *const *av)
{
    struct Display *dpy;
    int fd;

    if ((dpy = malloc(sizeof(struct Display))) == NULL)
	err(1, "malloc");
    fd = openSocket(hostport);
    Socket_setNoDelay(fd);

    if ((dpy->out = FileDescriptor_createOutputStream(fd, 0)).super.object
	== NULL) {
	errx(1, "failed to create OuputStream.");
    }
    if ((dpy->in = BufferedInputStream_create(
	     FileDescriptor_createInputStream(fd, 0))).super.object == NULL) {
	errx(1, "failed to create InputStream.");
    }
    dpy->fd = fd;
    dpy->lastEvent = NULL;
    dpy->testsuite = av;
    return dpy;
}

void
Display_close(struct Display *dpy)
{
    Event_free(dpy->lastEvent);
    InputStream_close(dpy->in.super.inputStream);
    OutputStream_close(dpy->out);
    close(dpy->fd);
    free(dpy);
}

void
Display_sendRecipe(struct Display *dpy)
{
    handshake(dpy->out, dpy->testsuite);
}

union Event *
Display_getEvent(struct Display *dpy)
{
    Event_free(dpy->lastEvent);
    dpy->lastEvent = Event_create(dpy, dpy->in.super.inputStream);
    return dpy->lastEvent;
}

void
Display_sendData(struct Display *dpy, void *data, size_t size)
{
    if (OutputStream_writeFully(dpy->out, data, size) < 0) {
	errx(1, "OutputStream_writeFully() failed.");
    }
}
