/*
	$Id: Funnel.c 232 2006-06-03 18:46:16Z syl $

	Copyright (C) 2006 Syllabub
	Maroontress Fast Software.
*/

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <err.h>
#include "Auditor.h"
#include "Display.h"
#include "Event.h"
#include "Test.h"

static void
usage(const char *progName)
{
    fprintf(stderr, "\
usage: %s [-s] [-t seconds] [-d hostname[:port]] solib [solib ...]\n",
	    progName);
    exit(1);
}

int
main(int ac, const char *const *av)
{
    struct Display *dpy;
    union Event *ev;
    const char *progName = av[0];
    const char *hostport = getenv("TF_DISPLAY");
    int standalone = 0, seconds = 5, opt;

    while ((opt = getopt(ac, (char *const *)av, "st:d:")) != -1) {
	switch (opt) {
	case 's':
	    standalone = 1;
	    break;
	case 't':
	    if ((seconds = atoi(optarg)) <= 0)
		errx(1, "invalid value: %s", optarg);
	    break;
	case 'd':
	    hostport = optarg;
	    break;
	default:
	    usage(progName);
	    /* NOTREAHCED */
	}
    }
    ac -= optind;
    av += optind;
    if (ac < 1) {
	usage(progName);
    }
    Auditor_setTimeout(seconds);

    if (standalone) {
	Test_invokeAll(av);
	exit(0);
    }
    dpy = Display_open(hostport, av);
    Display_sendRecipe(dpy);
    while ((ev = Display_getEvent(dpy)) != NULL) {
	Event_dispatch(ev);
    }
    Display_close(dpy);
    exit(0);
    return 0;
}
