/*
	$Id: Recipe.c 220 2006-05-31 13:58:03Z syl $

	Copyright (C) 2006 Syllabub
	Maroontress Fast Software.
*/

#include <stdio.h>
#include <err.h>
#include "TestListener.h"
#include "Test.h"
#include "Recipe.h"

static void
printUnsafeString(FILE *fp, const char *s)
{
    static const char
	gt[] = "&gt;", lt[] = "&lt;", amp[] = "&amp;", quot[] = "&quot;";
    const char *r;

    for (; *s; ++s) {
	switch (*s) {
	default:
	    putc(*s, fp);
	    continue;
	case '>':
	    r = gt;
	    break;
	case '<':
	    r = lt;
	    break;
	case '&':
	    r = amp;
	    break;
	case '"':
	    r = quot;
	    break;
	}
	fprintf(fp, "%s", r);
    }
}

static void
printAllClasses(FILE *fp, const char *const *av)
{
    auto void classOpened(const char *, const char *);
    auto void classClosed(void);
    auto void methodFound(const char *);
    struct TestListener listener = {
	.classOpened = classOpened,
	.classClosed = classClosed,
	.methodFound = methodFound};

    void classOpened(const char *className, const char *desc) {
	fprintf(fp, "  <class name=\"");
	printUnsafeString(fp, className);
	fprintf(fp, "\" description=\"");
	printUnsafeString(fp, desc);
	fprintf(fp, "\">\n");
    }

    void methodFound(const char *methodName) {
	fprintf(fp, "    <method name=\"");
	printUnsafeString(fp, methodName);
	fprintf(fp, "\"/>\n");
    }

    void classClosed(void) {
	fprintf(fp, "  </class>\n");
    }

    fprintf(fp,
	    "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
	    "<funnel>\n");
    for (; *av != NULL; ++av) {
	Test_findClass(*av, &listener);
    }
    fprintf(fp,
	    "</funnel>\n");
}

void
Recipe_print(union OutputStream out, const char *const *av)
{
    FILE *fp;

    if ((fp = OutputStream_getAncientStream(out)) == NULL)
	errx(1, "OutputStream_getAncientStream() failed.");
    printAllClasses(fp, av);
    fclose(fp);
}
